//
//  ViewController.m
//  Dropit
//
//  Created by Sameh Fakhouri on 10/24/16.
//  Copyright © 2016 Sameh Fakhouri. All rights reserved.
//

#import "ViewController.h"
#import "DropitBehavior.h"
#import "BezierPathView.h"

@interface ViewController () <UIDynamicAnimatorDelegate>
@property (weak, nonatomic) IBOutlet BezierPathView *gameView;

@property (strong, nonatomic) UIDynamicAnimator *animator;
@property (strong, nonatomic) DropitBehavior *dropitBehavior;


@property (strong, nonatomic) UIAttachmentBehavior *attachment;
@property (strong, nonatomic) UIView *droppingView;
@end

@implementation ViewController


- (UIDynamicAnimator *)animator {
    if (!_animator) {
        _animator = [[UIDynamicAnimator alloc] initWithReferenceView:self.gameView];
        _animator.delegate = self;
    }
    return _animator;
}


- (DropitBehavior *)dropitBehavior {
    if (!_dropitBehavior) {
        _dropitBehavior = [[DropitBehavior alloc] init];
        [self.animator addBehavior:_dropitBehavior];
    }
    return _dropitBehavior;
}

- (void)dynamicAnimatorDidPause:(UIDynamicAnimator *)animator {
    [self removeCompleteRows];
}


- (void)removeCompleteRows {
    NSMutableArray *dropsToRemove = [[NSMutableArray alloc] init];
    
    for ( CGFloat y = self.gameView.bounds.size.height - DROP_SIZE.height/2 ;
         y > 0 ;
         y -= DROP_SIZE.height) {
        
        BOOL rowIsComplete = YES;
        NSMutableArray *dropsFound = [[NSMutableArray alloc] init];
        
        for ( CGFloat x = DROP_SIZE.width/2 ; x < self.gameView.bounds.size.width ; x += DROP_SIZE.width ) {
            
            UIView *hitView = [self.gameView hitTest:CGPointMake(x, y) withEvent:NULL] ;
            
            if ([hitView superview] == self.gameView) {
                [dropsFound addObject:hitView];
            } else {
                rowIsComplete = NO;
                break;
            }
        }
        
        if (![dropsFound count]) {
            break;
        }
        
        if (rowIsComplete) {
            [dropsToRemove addObjectsFromArray:dropsFound];
        }
    }
    
    if ([dropsToRemove count]) {
        for (UIView *drop in dropsToRemove) {
            [self.dropitBehavior removeItem:drop];
        }
        [self animateRemovingDrops:dropsToRemove];
    }
}

- (void)animateRemovingDrops:(NSArray *)dropsToRemove {
    [UIView animateWithDuration:1.0
                     animations:^{
                         for (UIView *drop in dropsToRemove) {
                             int x = (arc4random() % (int)(self.gameView.bounds.size.width*5));
                             int y = (int)self.gameView.bounds.size.height;
                             drop.center = CGPointMake(x, -y);
                        }
                     }
                    completion:^(BOOL finished) {
                        [dropsToRemove makeObjectsPerformSelector:@selector(removeFromSuperview)];
                    }];
}



- (IBAction)tap:(UITapGestureRecognizer *)sender {
    [self drop];
}

static const CGSize DROP_SIZE = { 40, 40 };

- (void)drop {
    CGRect frame;
    frame.origin = CGPointZero;
    frame.size = DROP_SIZE;
    int x = (arc4random() % (int)self.gameView.bounds.size.width) / DROP_SIZE.width;
    frame.origin.x = x * DROP_SIZE.width;
    
    
    UIView *dropView = [[UIView alloc] initWithFrame:frame];
    dropView.backgroundColor = [self randomColor];
    
    self.droppingView = dropView;
    
    [self.gameView addSubview:dropView];
    [self.dropitBehavior addItem:dropView];
}

- (UIColor *)randomColor {
    UIColor *color;
    switch (arc4random() % 5) {
        case 0: color = [UIColor greenColor]; break;
        case 1: color = [UIColor blueColor]; break;
        case 2: color = [UIColor orangeColor]; break;
        case 3: color = [UIColor redColor]; break;
        case 4: color = [UIColor purpleColor]; break;
    }
    
    return color;
}

- (IBAction)pan:(UIPanGestureRecognizer *)sender {
    // get the location of the where the finger is located in the view
    CGPoint gesturePoint = [sender locationInView:self.gameView];
    if (sender.state == UIGestureRecognizerStateBegan) {
        [self attachDroppingViewToPoint:gesturePoint];
    } else if (sender.state == UIGestureRecognizerStateChanged) {
        self.attachment.anchorPoint = gesturePoint;
    } else if (sender.state == UIGestureRecognizerStateEnded) {
        [self.animator removeBehavior:self.attachment];
        self.gameView.path = nil;
    }
}

- (void)attachDroppingViewToPoint:(CGPoint)anchorPoint
{
    // do we have a droppingView?
    if (self.droppingView) {
        // yes, attach to it
        self.attachment = [[UIAttachmentBehavior alloc] initWithItem:self.droppingView
                                                    attachedToAnchor:anchorPoint];

        UIView *myDroppingView = self.droppingView;
        __weak ViewController *weakSelf = self;
        
        self.attachment.action = ^{
            UIBezierPath *path = [[UIBezierPath alloc] init];
            [path moveToPoint:weakSelf.attachment.anchorPoint];
            [path addLineToPoint:myDroppingView.center];
            weakSelf.gameView.path = path;
        };
        
        // once attached, you cannot re-grab it again. You only get one try
        self.droppingView = nil;
        
        // add the attachment behavior to the animator
        [self.animator addBehavior:self.attachment];
    }
    
}
























@end
